@@ -421,31 +421,70 @@ def get_groups(_=None, as_json=False, current_user=None):
     return groups
 
 
-def get_jobs(_=None, as_json=False, current_user=None, limit=None):
+def get_job(session, id, as_json=False):
+
     from seamm_datastore.database.models import Job, Project
+    from .util import NotAuthorizedError
 
-    jobs = Job.query.all()
+    authorized_job = Job.query.filter(Job.authorized("read"), Job.id == id)
+    project_job = Project.query.with_entities(Job).filter(
+        Project.authorized("read"), Job.id == id
+    )
 
-    # After we get the jobs, we need to know which jobs belong to projects which the
-    # user can read. This might be a performance issue on a larger DB, but I think we
-    # can do this check here.
-    authorized_jobs = Job.query.filter(Job.authorized("read")).all()
-    auth_projects = Project.query.filter(Project.authorized("read")).all()
-    for job in jobs:
-        if job not in authorized_jobs:
-            for project in job.projects:
-                if project in auth_projects:
-                    authorized_jobs.append(job)
-                    # We can exit the loop if we have found one
-                    # project which grants read permission
-                    break
+    project_job = Job.query.filter(
+        Job.projects.any(Project.authorized("read")), Job.id == id
+    )
+
+    # Do we get the job?
+    job = authorized_job.union(project_job).one_or_none()
+
+    if job is None and Job.query.get(id):
+        raise NotAuthorizedError
 
     if as_json:
         from seamm_datastore.database.schema import JobSchema
 
-        jobs = JobSchema(many=True).dump(authorized_jobs)
-    if limit:
-        jobs = jobs[:limit]
+        job = JobSchema().dump(job)
+
+    return job
+
+
+def get_jobs(_=None, as_json=False, limit=None, offset=None, count=False, status=None):
+
+    from seamm_datastore.database.models import Job, Project
+
+    # Get the jobs where the user has read permission
+    if status is None:
+        authorized_jobs = Job.query.filter(Job.authorized("read"))
+    else:
+        authorized_jobs = Job.query.filter(Job.authorized("read"), Job.status == status)
+
+    # Get jobs from projects where user has read permission
+    if status is None:
+        project_jobs = Job.query.filter(Job.projects.any(Project.authorized("read")))
+    else:
+        project_jobs = Job.query.filter(
+            Job.projects.any(Project.authorized("read")), Job.status == status
+        )
+
+    # Find union of these queries
+    jobs = authorized_jobs.union(project_jobs)
+
+    # Continue building
+    if limit is not None:
+        jobs = jobs.limit(limit)
+    if offset is not None:
+        jobs = jobs.offset(offset)
+
+    if count is True:
+        jobs = jobs.count()
+    else:
+        jobs = jobs.all()
+
+        if as_json:
+            from seamm_datastore.database.schema import JobSchema
+
+            jobs = JobSchema(many=True).dump(jobs)
 
     return jobs
 
